@@ -23,11 +23,11 @@ sub match_index{
 
 =head1 NAME
 
-TaskPipe::InterpParam::MatchHandler_this;
+TaskPipe::InterpParam::MatchHandler_this - match handler for the $this parameter variable
 
 =head1 DESCRIPTION
 
-MatchHandler for the $this parameter value
+MatchHandler for the $this parameter variable
 
 =cut
 
