@@ -13,7 +13,7 @@ subtype 'OutputMode',
 
 =head1 NAME
 
-TaskPipe::Role::MooseType_OutputMode
+TaskPipe::Role::MooseType_OutputMode - output mode type constraint
 
 =head1 DESCRIPTION
 
