@@ -13,7 +13,7 @@ subtype 'PlanMode',
 
 =head1 NAME
 
-TaskPipe::Role::MooseType_PlanMode
+TaskPipe::Role::MooseType_PlanMode - plan mode type constraint
 
 =head1 DESCRIPTION
 
