@@ -34,6 +34,27 @@ sub target_dir{
     );
 }
 
+=head1 NAME
+
+TaskPipe::Template_Task - base class for task templates
+
+=head1 DESCRIPTION
+
+Base class for task templates
+
+=head1 AUTHOR
+
+Tom Gracey <tomgracey@gmail.com>
+
+=head1 COPYRIGHT AND LICENSE
+
+Copyright (c) Tom Gracey 2018
+
+TaskPipe is free software, licensed under
+
+    The GNU Public License Version 3
+
+=cut
 
 
 
